@@ -14,12 +14,14 @@
 #include <CL/sycl.hpp>
 #include "util.h"
 
-#define OUTPUT_WIDTH      256
-#define OUTPUT_HEIGHT     192
-#define OUTPUT_FILE       "out.raw"
-#define BLUR_RADIUS       5
-#define BLUR_SIZE         (float)((BLUR_RADIUS << 1) + 1)
-#define MAX_PLANES_NUMBER 4
+#define OUTPUT_WIDTH               256
+#define OUTPUT_HEIGHT              192
+#define OUTPUT_FILE                "out.raw"
+#define BLUR_RADIUS                5
+#define BLUR_SIZE                  (float)((BLUR_RADIUS << 1) + 1)
+#define MAX_PLANES_NUMBER          4
+#define MAJOR_API_VERSION_REQUIRED 2
+#define MINOR_API_VERSION_REQUIRED 2
 
 #ifdef LIBVA_SUPPORT
     #define HAVE_VIDEO_MEMORY_INTEROP
@@ -85,7 +87,7 @@ void Usage(char *app) {
     printf("     -i input file name (sw=I420 raw frames, hw=NV12)\n");
     printf("     -w input width\n");
     printf("     -h input height\n\n");
-    printf("   Example:  %s -i in.i420 -w 128 -h 96\n", app);
+    printf("   Example:  %s -i in.i420 -w 320 -h 240\n", app);
     printf("   To view:  ffplay -f rawvideo -video_size %dx%d "
            "-pixel_format bgra %s\n\n",
            OUTPUT_WIDTH,
@@ -107,9 +109,11 @@ void Usage(char *app) {
 int main(int argc, char **argv) {
     bool isDraining   = false;
     bool isStillGoing = true;
+    bool isFailed     = false;
     FILE *sink        = NULL;
     FILE *source      = NULL;
-    mfxConfig cfg[1];
+    mfxConfig cfg[2];
+    mfxVariant cfgVal[2];
     mfxLoader loader   = NULL;
     mfxSession session = NULL;
     mfxStatus sts      = MFX_ERR_NONE;
@@ -156,15 +160,19 @@ int main(int argc, char **argv) {
            q.get_device().get_info<sycl::info::device::name>().c_str());
 
 #ifdef HAVE_VIDEO_MEMORY_INTEROP
-    // Get Level-zero context and device from the SYCL backend
-    ze_context_handle_t ze_context = q.get_context().get_native<sycl::backend::level_zero>();
+    ze_context_handle_t ze_context = {};
+    ze_device_handle_t ze_device   = {};
+    if (MFX_IMPL_TYPE_SOFTWARE != cliParams.implValue.Data.U32) {
+        // Get Level-zero context and device from the SYCL backend
+        ze_context = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(q.get_context());
+        ze_device  = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(q.get_device());
 
-    ze_device_handle_t ze_device = q.get_device().get_native<sycl::backend::level_zero>();
-    if (q.get_device().get_info<sycl::info::device::device_type>() ==
-        sycl::info::device_type::gpu) {
-        ze_device_properties_t deviceProperties;
-        zeDeviceGetProperties(ze_device, &deviceProperties);
-        desiredDevice = (mfxU32)deviceProperties.deviceId;
+        if (q.get_device().get_info<sycl::info::device::device_type>() ==
+            sycl::info::device_type::gpu) {
+            ze_device_properties_t deviceProperties;
+            zeDeviceGetProperties(ze_device, &deviceProperties);
+            desiredDevice = (mfxU32)deviceProperties.deviceId;
+        }
     }
 #endif
 
@@ -191,6 +199,16 @@ int main(int argc, char **argv) {
         MFXSetConfigFilterProperty(cfg[0], (mfxU8 *)"mfxImplDescription.Impl", cliParams.implValue);
     VERIFY(MFX_ERR_NONE == sts, "MFXSetConfigFilterProperty failed for Impl");
 
+    // Implementation used must provide API version 2.2 or newer
+    cfg[1] = MFXCreateConfig(loader);
+    VERIFY(NULL != cfg[1], "MFXCreateConfig failed")
+    cfgVal[1].Type     = MFX_VARIANT_TYPE_U32;
+    cfgVal[1].Data.U32 = VPLVERSION(MAJOR_API_VERSION_REQUIRED, MINOR_API_VERSION_REQUIRED);
+    sts                = MFXSetConfigFilterProperty(cfg[1],
+                                     (mfxU8 *)"mfxImplDescription.ApiVersion.Version",
+                                     cfgVal[1]);
+    VERIFY(MFX_ERR_NONE == sts, "MFXSetConfigFilterProperty failed for API version");
+
     // Match oneVPL implementation with sycl::q device
     while (MFX_ERR_NOT_FOUND != MFXEnumImplementations(loader,
                                                        sessionIdx,
@@ -474,5 +492,10 @@ end:
     }
 #endif
 
-    return 0;
+    if (isFailed) {
+        return -1;
+    }
+    else {
+        return 0;
+    }
 }
