@@ -187,6 +187,83 @@ inline mfxU16 ReadFrameType(const std::string& line) {
 }
 } // namespace QPFile
 
+namespace TCBRCTestFile {
+
+enum ReaderStatus {
+    READER_ERR_NONE,
+    READER_ERR_NOT_INITIALIZED,
+    READER_ERR_CODEC_UNSUPPORTED,
+    READER_ERR_FILE_NOT_OPEN,
+    READER_ERR_INCORRECT_FILE
+};
+
+struct FrameInfo {
+    mfxU32 displayOrder;
+    mfxU32 targetFrameSize;
+};
+
+// TCBRCTestFile reads target frame size in display order
+// from external text file (for encoding in Low delay BRC mode)
+class Reader {
+public:
+    mfxStatus Read(const msdk_string& strFileName, mfxU32 codecid);
+    void ResetState();
+
+    mfxU32 GetTargetFrameSize(mfxU32 frameOrder) const;
+    mfxU32 GetFramesNum() const;
+    void NextFrame();
+    std::string GetErrorMessage() const;
+
+private:
+    void ResetState(ReaderStatus set_sts);
+
+    ReaderStatus m_ReaderSts = READER_ERR_NOT_INITIALIZED;
+    mfxU32 m_CurFrameNum     = std::numeric_limits<mfxU32>::max();
+    std::vector<FrameInfo> m_FrameVals{};
+};
+
+inline bool get_line(std::ifstream& ifs, std::string& line) {
+    std::getline(ifs, line, '\n');
+    if (!line.empty() && line.back() == '\r')
+        line.pop_back();
+    return !ifs.fail();
+}
+inline size_t find_nth(const std::string& str, size_t pos, const std::string& needle, mfxU32 nth) {
+    size_t found_pos = str.find(needle, pos);
+    for (; nth != 0 && std::string::npos != found_pos; --nth)
+        found_pos = str.find(needle, found_pos + 1);
+    return found_pos;
+}
+inline std::string ReaderStatusToString(ReaderStatus sts) {
+    switch (sts) {
+        case READER_ERR_NOT_INITIALIZED:
+            return std::string(
+                "reader not initialized (TCBRCTestfile has not yet read the file)\n");
+        case READER_ERR_FILE_NOT_OPEN:
+            return std::string(
+                "failed to open file  with TargetFrameSize parameters (check provided path in -tcbrcfile <path>)\n");
+        case READER_ERR_INCORRECT_FILE:
+            return std::string("incorrect file with frame parameters\n");
+        case READER_ERR_CODEC_UNSUPPORTED:
+            return std::string("h264 and h265 are supported now\n");
+        default:
+            return std::string();
+    }
+}
+inline mfxU32 ReadDisplayOrder(const std::string& line) {
+    size_t pos = find_nth(line, 0, ":", 0);
+    if (pos != std::string::npos)
+        return std::stoi(line.substr(0, pos));
+    else
+        return 0;
+}
+inline mfxU16 ReadTargetFrameSize(const std::string& line) {
+    size_t pos = find_nth(line, 0, ":", 0);
+    pos        = (pos != std::string::npos) ? pos + 1 : 0;
+    return static_cast<mfxU16>(std::stoi(line.substr(pos, line.size() - pos)));
+}
+} // namespace TCBRCTestFile
+
 mfxStatus GetFrameLength(mfxU16 width, mfxU16 height, mfxU32 ColorFormat, mfxU32& length);
 
 bool IsDecodeCodecSupported(mfxU32 codecFormat);
@@ -229,6 +306,10 @@ struct mfx_ext_buffer_id<mfxExtAvcTemporalLayers> {
     enum { id = MFX_EXTBUFF_AVC_TEMPORAL_LAYERS };
 };
 template <>
+struct mfx_ext_buffer_id<mfxExtTemporalLayers> {
+    enum { id = MFX_EXTBUFF_UNIVERSAL_TEMPORAL_LAYERS };
+};
+template <>
 struct mfx_ext_buffer_id<mfxExtAVCRefListCtrl> {
     enum { id = MFX_EXTBUFF_AVC_REFLIST_CTRL };
 };
@@ -384,6 +465,19 @@ template <>
 struct mfx_ext_buffer_id<mfxExtAllocationHints> {
     enum { id = MFX_EXTBUFF_ALLOCATION_HINTS };
 };
+template <>
+struct mfx_ext_buffer_id<mfxExtVPP3DLut> {
+    enum { id = MFX_EXTBUFF_VPP_3DLUT };
+};
+
+template <>
+struct mfx_ext_buffer_id<mfxExtMasteringDisplayColourVolume> {
+    enum { id = MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME };
+};
+template <>
+struct mfx_ext_buffer_id<mfxExtContentLightLevelInfo> {
+    enum { id = MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO };
+};
 
 constexpr uint16_t max_num_ext_buffers =
     63 * 2; // '*2' is for max estimation if all extBuffer were 'paired'
@@ -454,7 +548,7 @@ public:
 
     ExtBufHolder& operator=(const ExtBufHolder& ref) {
         const T* src_base = &ref;
-        return operator   =(*src_base);
+        return operator=(*src_base);
     }
 
     ExtBufHolder(const T& ref) {
@@ -492,7 +586,7 @@ public:
         return *this;
     }
 
-    ExtBufHolder(ExtBufHolder&&) = default;
+    ExtBufHolder(ExtBufHolder&&)            = default;
     ExtBufHolder& operator=(ExtBufHolder&&) = default;
 
     // Always returns a valid pointer or throws an exception
@@ -666,7 +760,7 @@ public:
         return *this;
     }
 
-    mfxBitstreamWrapper(mfxBitstreamWrapper&& bs_wrapper) = default;
+    mfxBitstreamWrapper(mfxBitstreamWrapper&& bs_wrapper)            = default;
     mfxBitstreamWrapper& operator=(mfxBitstreamWrapper&& bs_wrapper) = default;
     ~mfxBitstreamWrapper()                                           = default;
 
@@ -674,7 +768,7 @@ public:
         if (MaxLength >= n_bytes)
             return;
 
-        m_data.resize(n_bytes);
+        m_data.reserve(n_bytes);
 
         Data      = m_data.data();
         MaxLength = n_bytes;
@@ -696,6 +790,7 @@ public:
                            bool shouldShiftP010 = false);
     virtual mfxStatus SkipNframesFromBeginning(mfxU16 w, mfxU16 h, mfxU32 viewId, mfxU32 nframes);
     virtual mfxStatus LoadNextFrame(mfxFrameSurface1* pSurface);
+    virtual mfxStatus LoadNextFrame(mfxFrameSurface1* pSurface, int bytes_to_read, mfxU8* buf_read);
     virtual void Reset();
     mfxU32 m_ColorFormat; // color format of input YUV data, YUV420 or NV12
 
@@ -716,6 +811,7 @@ public:
     virtual mfxStatus WriteNextFrame(mfxBitstream* pMfxBitstream,
                                      bool isPrint         = true,
                                      bool isCompleteFrame = true);
+    virtual mfxStatus WriteNextFrame(mfxBitstream* pMfxBitstream, mfxU32 targetID, mfxU32 frameNum);
     virtual mfxStatus Reset();
     virtual void Close();
     mfxU32 m_nProcessedFramesNum;
@@ -1130,6 +1226,52 @@ struct MSDKAdapter {
 
         return adapterNum;
     }
+
+    // returns the number of adapter associated with MSDK session, 0 for SW session
+    static mfxU32 GetNumber(mfxSession session, mfxIMPL implVia = 0) {
+        mfxU32 adapterNum = 0; // default
+        mfxIMPL impl      = MFX_IMPL_SOFTWARE; // default in case no HW IMPL is found
+
+        // we don't care for error codes in further code; if something goes wrong we fall back to the default adapter
+        if (session) {
+            MFXQueryIMPL(session, &impl);
+        }
+        else {
+            // an auxiliary session, internal for this function
+            mfxSession auxSession;
+            memset(&auxSession, 0, sizeof(auxSession));
+
+            mfxVersion ver = { { 1, 1 } }; // minimum API version which supports multiple devices
+            MFXInit(MFX_IMPL_HARDWARE_ANY | implVia, &ver, &auxSession);
+            MFXQueryIMPL(auxSession, &impl);
+            MFXClose(auxSession);
+        }
+
+        // extract the base implementation type
+        mfxIMPL baseImpl = MFX_IMPL_BASETYPE(impl);
+
+        const struct {
+            // actual implementation
+            mfxIMPL impl;
+            // adapter's number
+            mfxU32 adapterID;
+
+        } implTypes[] = { { MFX_IMPL_HARDWARE, 0 },
+                          { MFX_IMPL_SOFTWARE, 0 },
+                          { MFX_IMPL_HARDWARE2, 1 },
+                          { MFX_IMPL_HARDWARE3, 2 },
+                          { MFX_IMPL_HARDWARE4, 3 } };
+
+        // get corresponding adapter number
+        for (mfxU8 i = 0; i < sizeof(implTypes) / sizeof(*implTypes); i++) {
+            if (implTypes[i].impl == baseImpl) {
+                adapterNum = implTypes[i].adapterID;
+                break;
+            }
+        }
+
+        return adapterNum;
+    }
 };
 
 struct APIChangeFeatures {
@@ -1278,6 +1420,8 @@ private:
                  it++) {
                 mfxU32 refCount = -1;
                 mfxStatus sts   = (*it)->FrameInterface->GetRefCounter((*it), &refCount);
+                // 2 means that only library and synchronizer have control over surface
+                // and all components don't use it now
                 if (sts == MFX_ERR_NONE && refCount <= 2 && (*it)->Data.Locked == 0) {
                     (*it)->FrameInterface->Release((*it));
                     m_Surfaces.erase(it);
@@ -1394,4 +1538,22 @@ protected:
     void UpdateNumberOfFrames();
 };
 
+class CBitstreamWriterForParallelEncoding : public CSmplBitstreamWriter {
+public:
+    virtual mfxStatus WriteNextFrame(mfxBitstream* pMfxBitstream, mfxU32 targetID, mfxU32 frameNum);
+    virtual mfxStatus Reset();
+
+    mfxU32 m_GopSize          = 0;
+    mfxU32 m_NumberOfEncoders = 0;
+    mfxU32 m_BaseEncoderID    = 0;
+    bool m_WriteBsToStdout    = false;
+
+private:
+    int m_LastWrittenFrameNumber = -1;
+    std::mutex m_Mutex{};
+    std::map<mfxU32, std::vector<mfxU8>> m_Buffer{}; //key is targetID
+    std::map<mfxU32, mfxU32> m_FirstFrameInBuffer{}; //key is target ID
+    std::map<mfxU32, mfxU32> m_LastFrameInBuffer{}; //key is target ID
+};
+
 #endif //__SAMPLE_UTILS_H__
