@@ -82,6 +82,8 @@ mfxStatus cBRCParams::Init(mfxVideoParam* par, bool bField) {
             HRDConformance = MFX_BRC_HRD_WEAK;
     }
 
+    bufferSizeInBytes =
+        (k * par->mfx.BufferSizeInKB * 1000); // Bitstream size is Buffer size if specified
     if (HRDConformance != MFX_BRC_NO_HRD) {
         bufferSizeInBytes = ((k * par->mfx.BufferSizeInKB * 1000) >> (cpb_size_scale + 1))
                             << (cpb_size_scale + 1);
@@ -608,6 +610,13 @@ mfxStatus ExtBRC::Update(mfxBRCFrameParam* frame_par,
         e2pe = (m_ctx.eRate == 0) ? (BRC_SCENE_CHANGE_RATIO2 + 1) : eRate / m_ctx.eRate;
 
     mfxU32 frameSizeLim = 0xfffffff; // sliding window limitation or external frame size limitation
+    // HEVC / AV1 do not have external re-encode loop to prevent bitstream copy size violation.
+    // For NO_HRD this will ensure encoded picture fits in bitstream memory.
+    // Bitstsream size is bufferSize if specified.
+    mfxU32 bitstreamSizeLim = frameSizeLim;
+    if (m_par.codecId != MFX_CODEC_AVC && m_par.HRDConformance == MFX_BRC_NO_HRD &&
+        m_par.bufferSizeInBytes)
+        bitstreamSizeLim = m_par.bufferSizeInBytes * 8;
 
     bool bSHStart      = false;
     bool bNeedUpdateQP = false;
@@ -700,9 +709,10 @@ mfxStatus ExtBRC::Update(mfxBRCFrameParam* frame_par,
     // Check other condions for recoding (update qp is it is needed)
     {
         mfxF64 targetFrameSize = std::max<mfxF64>((mfxF64)m_par.inputBitsPerFrame, fAbLong);
-        mfxF64 maxFrameSize =
-            (m_ctx.encOrder == 0 ? 6.0 : (bSHStart || picType == MFX_FRAMETYPE_I) ? 8.0 : 4.0) *
-            targetFrameSize * (m_par.bPyr ? 1.5 : 1.0);
+        mfxF64 maxFrameSize    = (m_ctx.encOrder == 0                        ? 6.0
+                                  : (bSHStart || picType == MFX_FRAMETYPE_I) ? 8.0
+                                                                             : 4.0) *
+                              targetFrameSize * (m_par.bPyr ? 1.5 : 1.0);
         mfxI32 quantMax = m_ctx.QuantMax;
         mfxI32 quantMin = m_ctx.QuantMin;
         mfxI32 quant    = qpY;
@@ -812,9 +822,10 @@ mfxStatus ExtBRC::Update(mfxBRCFrameParam* frame_par,
             }
         } //m_par.HRDConformance
     }
-    if (((m_par.HRDConformance != MFX_BRC_NO_HRD && brcSts != MFX_BRC_OK) ||
-         (bitsEncoded > (mfxI32)frameSizeLim)) &&
-        m_par.bRec) {
+    if ((((m_par.HRDConformance != MFX_BRC_NO_HRD && brcSts != MFX_BRC_OK) ||
+          (bitsEncoded > (mfxI32)frameSizeLim)) &&
+         m_par.bRec) ||
+        (bitsEncoded > (mfxI32)bitstreamSizeLim)) {
         mfxI32 quant     = m_ctx.Quant;
         mfxI32 quant_new = quant;
         if (bitsEncoded > (mfxI32)frameSizeLim) {
@@ -828,6 +839,17 @@ mfxStatus ExtBRC::Update(mfxBRCFrameParam* frame_par,
                                  1,
                                  true);
         }
+        else if (bitsEncoded > (mfxI32)bitstreamSizeLim) {
+            brcSts    = MFX_BRC_BIG_FRAME;
+            quant_new = GetNewQP(bitsEncoded,
+                                 bitstreamSizeLim,
+                                 m_ctx.QuantMin,
+                                 m_ctx.QuantMax,
+                                 quant,
+                                 m_par.quantOffset,
+                                 1,
+                                 true);
+        }
         else if (brcSts == MFX_BRC_BIG_FRAME || brcSts == MFX_BRC_SMALL_FRAME) {
             mfxF64 targetSize = GetFrameTargetSize(
                 brcSts,
@@ -1053,8 +1075,9 @@ mfxStatus ExtBRC::Reset(mfxVideoParam* par) {
             sts = m_par.Init(par, isFieldMode(par));
             MFX_CHECK_STS(sts);
 
-            m_ctx.Quant = (mfxI32)(
-                1. / m_ctx.dQuantAb * pow(m_ctx.fAbLong / m_par.inputBitsPerFrame, 0.32) + 0.5);
+            m_ctx.Quant =
+                (mfxI32)(1. / m_ctx.dQuantAb * pow(m_ctx.fAbLong / m_par.inputBitsPerFrame, 0.32) +
+                         0.5);
             m_ctx.Quant = mfx::clamp(m_ctx.Quant, m_par.quantMinI, m_par.quantMaxI);
 
             UpdateQPParams(m_ctx.Quant,
@@ -1244,12 +1267,10 @@ mfxU32 H264_HRD::GetInitCpbRemovalDelay(mfxU32 /* eo */) const {
     double delay                  = std::max(0.0, m_trn_cur - m_taf_prv);
     mfxU32 initialCpbRemovalDelay = mfxU32(90000 * delay + 0.5);
 
-    return (mfxU32)(initialCpbRemovalDelay == 0
-                        ? 1 // should not be equal to 0
-                        : initialCpbRemovalDelay > m_hrdInput.m_cpbSize90k &&
-                                  (!m_hrdInput.m_cbrFlag)
-                              ? m_hrdInput.m_cpbSize90k // should not exceed hrd buffer
-                              : initialCpbRemovalDelay);
+    return (mfxU32)(initialCpbRemovalDelay == 0 ? 1 // should not be equal to 0
+                    : initialCpbRemovalDelay > m_hrdInput.m_cpbSize90k && (!m_hrdInput.m_cbrFlag)
+                        ? m_hrdInput.m_cpbSize90k // should not exceed hrd buffer
+                        : initialCpbRemovalDelay);
 }
 mfxF64 H264_HRD::GetBufferDeviation(mfxU32 eo) const {
     mfxU32 delay       = GetInitCpbRemovalDelay(eo);
