@@ -129,7 +129,7 @@ static mfxStatus GetVAFourcc(mfxU32 fourcc, unsigned int& va_fourcc) {
     mfxU32 mfx_fourcc = ConvertVP8FourccToMfxFourcc(fourcc);
     va_fourcc         = ConvertMfxFourccToVAFormat(mfx_fourcc);
     if (!va_fourcc || ((VA_FOURCC_NV12 != va_fourcc) && (VA_FOURCC_YV12 != va_fourcc) &&
-                       (VA_FOURCC_YUY2 != va_fourcc) && (VA_FOURCC_ARGB != va_fourcc) &&
+                       (VA_FOURCC_UYVY != va_fourcc) && (VA_FOURCC_ARGB != va_fourcc) &&
                        (VA_FOURCC_ABGR != va_fourcc) && (VA_FOURCC_P208 != va_fourcc) &&
                        (VA_FOURCC_P010 != va_fourcc) && (VA_FOURCC_YUY2 != va_fourcc) &&
                        (VA_FOURCC_Y210 != va_fourcc) && (VA_FOURCC_Y410 != va_fourcc) &&
@@ -268,6 +268,9 @@ mfxStatus vaapiFrameAllocator::AllocImpl(mfxFrameAllocRequest* request,
             else if (va_fourcc == VA_FOURCC_NV12) {
                 format = VA_RT_FORMAT_YUV420;
             }
+            else if (va_fourcc == VA_FOURCC_P010) {
+                format = VA_RT_FORMAT_YUV420_10;
+            }
             else if ((va_fourcc == VA_FOURCC_UYVY) || (va_fourcc == VA_FOURCC_YUY2)) {
                 format = VA_RT_FORMAT_YUV422;
             }
@@ -690,5 +693,103 @@ mfxStatus vaapiFrameAllocator::GetFrameHDL(mfxMemId mid, mfxHDL* handle) {
     *handle = vaapi_mid->m_surface; //VASurfaceID* <-> mfxHDL
     return MFX_ERR_NONE;
 }
+mfxStatus vaapiFrameAllocator::Create3DLutMemory(mfxMemId memId, const char* lut3d_file_name) {
+    printf("vaapiFrameAllocator::Create3DLutMemory!\n");
+    VAStatus va_status    = VA_STATUS_SUCCESS;
+    VAImage surface_image = {};
+    void* surface_p       = NULL;
+    mfxU32 frame_size = 0, lut3d_size = 0;
+    mfxU8* newImageBuffer  = NULL;
+    VASurfaceID surface_id = VA_INVALID_ID;
+    mfxU32 seg_size = 65, mul_size = 128;
 
+    if (!memId) {
+        return MFX_ERR_NULL_PTR;
+    }
+
+    // create VA surface
+    VASurfaceAttrib surface_attrib = {};
+    surface_attrib.type            = VASurfaceAttribPixelFormat;
+    surface_attrib.flags           = VA_SURFACE_ATTRIB_SETTABLE;
+    surface_attrib.value.type      = VAGenericValueTypeInteger;
+    surface_attrib.value.value.i   = VA_FOURCC_RGBA;
+
+    va_status = m_libva->vaCreateSurfaces(m_dpy,
+                                          VA_RT_FORMAT_RGB32,
+                                          seg_size * mul_size,
+                                          seg_size * 2,
+                                          &surface_id,
+                                          1,
+                                          &surface_attrib,
+                                          1);
+    if (va_status != VA_STATUS_SUCCESS) {
+        printf("Load3DLutVAAPI vaCreateSurfaces 3dlut surface failed\n");
+        return MFX_ERR_UNSUPPORTED;
+    }
+
+    va_status = m_libva->vaSyncSurface(m_dpy, surface_id);
+    if (va_status != VA_STATUS_SUCCESS) {
+        printf("Load3DLutVAAPI vaSyncSurface 3dlut surface failed\n");
+        return MFX_ERR_UNSUPPORTED;
+    }
+
+    va_status = m_libva->vaDeriveImage(m_dpy, surface_id, &surface_image);
+    if (va_status != VA_STATUS_SUCCESS) {
+        printf("Load3DLutVAAPI vaDeriveImage from 3dlut surface failed\n");
+        return MFX_ERR_UNSUPPORTED;
+    }
+
+    va_status = m_libva->vaMapBuffer(m_dpy, surface_image.buf, &surface_p);
+    if (va_status != VA_STATUS_SUCCESS) {
+        printf("Load3DLutVAAPI vaMapBuffer from 3dlut surface failed\n");
+        return MFX_ERR_UNSUPPORTED;
+    }
+
+    if (surface_image.format.fourcc == VA_FOURCC_RGBA) {
+        /* 3DLUT surface is allocated to 32 bit RGB */
+        frame_size     = surface_image.width * surface_image.height * 4;
+        newImageBuffer = (mfxU8*)malloc(frame_size);
+        if (newImageBuffer) {
+            FILE* f3dlut = NULL;
+            f3dlut       = fopen(lut3d_file_name, "rb");
+            if (f3dlut) {
+                fseek(f3dlut, 0L, SEEK_END);
+                lut3d_size = ftell(f3dlut);
+                rewind(f3dlut);
+                uint32_t real_size = (frame_size > lut3d_size) ? lut3d_size : frame_size;
+                uint32_t read_size = fread(newImageBuffer, 1, real_size, f3dlut);
+                memcpy(surface_p, newImageBuffer, read_size);
+                printf(
+                    "upload_data_to_3dlut: 3DLUT surface width %d, height %d, pitch %d, frame size %d, 3dlut file size: %d\n",
+                    surface_image.width,
+                    surface_image.height,
+                    surface_image.pitches[0],
+                    frame_size,
+                    read_size);
+                fclose(f3dlut);
+                f3dlut = NULL;
+            }
+            free(newImageBuffer);
+            newImageBuffer = NULL;
+        }
+    }
+    m_libva->vaUnmapBuffer(m_dpy, surface_image.buf);
+    m_libva->vaDestroyImage(m_dpy, surface_image.image_id);
+
+    *((VASurfaceID*)memId) = surface_id;
+
+    printf("create 3dlut surface ID %d!\n", surface_id);
+    return MFX_ERR_NONE;
+}
+
+mfxStatus vaapiFrameAllocator::Release3DLutMemory(mfxMemId memId) {
+    VAStatus va_status = VA_STATUS_SUCCESS;
+    mfxStatus mfx_res  = MFX_ERR_NONE;
+
+    VASurfaceID surface_id = *((VASurfaceID*)memId);
+    va_status              = m_libva->vaDestroySurfaces(m_dpy, &surface_id, 1);
+    mfx_res                = va_to_mfx_status(va_status);
+
+    return mfx_res;
+}
 #endif // #if defined(LIBVA_SUPPORT)
