@@ -57,14 +57,20 @@ void PrintHelp(msdk_char* strAppName, const msdk_char* strErrorMessage) {
     msdk_printf(MSDK_STRING(
         "   [-scaling_mode value]     - specify scaling mode (lowpower/quality) for VPP\n"));
     msdk_printf(MSDK_STRING("   [-d]                      - enable decode error report\n"));
+#if defined(_WIN32) || defined(_WIN64)
     msdk_printf(MSDK_STRING(
-        "   [-dGfx]                   - preffer processing on dGfx (by default system decides), also can be set with index, for example: '-dGfx 1'\n"));
+        "   [-dxgiFs]                 - enable fullscreen via IDXGISwapChain::SetFullScreenState method\n"));
+#endif
     msdk_printf(MSDK_STRING(
-        "   [-iGfx]                   - preffer processing on iGfx (by default system decides)\n"));
+        "   [-dGfx]                   - prefer processing on dGfx (by default system decides), also can be set with index, for example: '-dGfx 1'\n"));
     msdk_printf(MSDK_STRING(
-        "   [-AdapterNum]             - specifies adpter number for processing, starts from 0\n"));
+        "   [-iGfx]                   - prefer processing on iGfx (by default system decides)\n"));
+    msdk_printf(MSDK_STRING(
+        "   [-AdapterNum]             - specifies adapter number for processing, starts from 0\n"));
     msdk_printf(MSDK_STRING(
         "   [-dispatcher:fullSearch]  - enable search for all available implementations in oneVPL dispatcher\n"));
+    msdk_printf(MSDK_STRING(
+        "   [-dispatcher:lowLatency]  - enable limited implementation search and query in oneVPL dispatcher\n"));
 #if defined(LINUX32) || defined(LINUX64)
     msdk_printf(MSDK_STRING("   [-device /path/to/device] - set graphics device for processing\n"));
     msdk_printf(
@@ -86,7 +92,9 @@ void PrintHelp(msdk_char* strAppName, const msdk_char* strErrorMessage) {
     msdk_printf(MSDK_STRING("   [-ignore_level_constrain] - ignore level constrain\n"));
 #endif
     msdk_printf(MSDK_STRING(
-        "   [-disable_film_grain] - disable film grain application(valid only for av1)\n"));
+        "   [-disable_film_grain]     - disable film grain application(valid only for av1)\n"));
+    msdk_printf(MSDK_STRING(
+        "   [-api_ver_init::<1x,2x>]  - select the api version for the session initialization\n"));
     msdk_printf(MSDK_STRING("\n"));
     msdk_printf(MSDK_STRING("JPEG Chroma Type:\n"));
     msdk_printf(MSDK_STRING("   [-jpeg_rgb] - RGB Chroma Type\n"));
@@ -179,6 +187,9 @@ void PrintHelp(msdk_char* strAppName, const msdk_char* strErrorMessage) {
 #if defined(_WIN32) || defined(_WIN64)
     msdk_printf(MSDK_STRING("   [-jpeg_rotate n]          - rotate jpeg frame n degrees \n"));
     msdk_printf(MSDK_STRING("       n(90,180,270)         - number of degrees \n"));
+#endif
+
+#if defined(_WIN32) || defined(_WIN64)
     msdk_printf(MSDK_STRING("\nFeatures: \n"));
     msdk_printf(MSDK_STRING("   Press 1 to toggle fullscreen rendering on/off\n"));
 #endif
@@ -202,7 +213,7 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
     pParams->adapterType        = mfxMediaAdapterType::MFX_MEDIA_UNKNOWN;
     pParams->dGfxIdx            = -1;
     pParams->adapterNum         = -1;
-    pParams->dispFullSearch     = false;
+    pParams->dispFullSearch     = DEF_DISP_FULLSEARCH;
 
 #if defined(LIBVA_SUPPORT)
     pParams->libvaBackend = MFX_LIBVA_DRM;
@@ -555,9 +566,15 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
                     return MFX_ERR_UNSUPPORTED;
                 }
             }
+#if (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+            pParams->bPreferdGfx = true;
+#endif
         }
         else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-iGfx"))) {
             pParams->adapterType = mfxMediaAdapterType::MFX_MEDIA_INTEGRATED;
+#if (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+            pParams->bPreferiGfx = true;
+#endif
         }
         else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-AdapterNum"))) {
             if (i + 1 >= nArgNum) {
@@ -572,6 +589,14 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
         else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-dispatcher:fullSearch"))) {
             pParams->dispFullSearch = true;
         }
+        else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-dispatcher:lowLatency"))) {
+            pParams->dispFullSearch = false;
+        }
+        else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-dxgiFs"))) {
+#if defined(_WIN32) || defined(_WIN64)
+            pParams->bDxgiFs = true;
+#endif
+        }
 #if !defined(_WIN32) && !defined(_WIN64)
         else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-d"))) {
             pParams->bErrorReport = true;
@@ -728,6 +753,12 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
         else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-disable_film_grain"))) {
             pParams->bDisableFilmGrain = true;
         }
+        else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-api_ver_init::1x"))) {
+            pParams->verSessionInit = API_1X;
+        }
+        else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-api_ver_init::2x"))) {
+            pParams->verSessionInit = API_2X;
+        }
         else // 1-character options
         {
             switch (strInput[i][1]) {
@@ -792,6 +823,13 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
         pParams->nAsyncDepth = 1;
     }
 
+#if (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+    if (pParams->bPreferdGfx && pParams->bPreferiGfx) {
+        msdk_printf(MSDK_STRING("Warning: both dGfx and iGfx flags set. iGfx will be preferred"));
+        pParams->bPreferdGfx = false;
+    }
+#endif
+
     return MFX_ERR_NONE;
 }
 
