@@ -146,10 +146,38 @@ enum MemoryModel {
     HIDDEN_INT_ALLOC  = 3
 };
 
+// the default api version is the latest one
+// it is located at 0
+enum eAPIVersion { API_2X, API_1X };
+
+typedef struct {
+    mfxU16 InsertPayloadToggle;
+    mfxU16 DisplayPrimariesX[3];
+    mfxU16 DisplayPrimariesY[3];
+    mfxU16 WhitePointX;
+    mfxU16 WhitePointY;
+    mfxU32 MaxDisplayMasteringLuminance;
+    mfxU32 MinDisplayMasteringLuminance;
+} sSEIMetaMasteringDisplay;
+
+typedef struct {
+    mfxU16 InsertPayloadToggle;
+    mfxU16 MaxContentLightLevel;
+    mfxU16 MaxPicAverageLightLevel;
+} sSEIMetaContentLightLevel;
+typedef struct {
+    bool Enabled;
+    mfxU16 VideoFullRange;
+    mfxU16 ColourPrimaries;
+} sVideoSignalInfo;
+
 struct __sInputParams {
-    mfxU32 TargetID    = 0;
-    bool CascadeScaler = false;
-    bool EnableTracing = false;
+    mfxU32 TargetID                    = 0;
+    bool CascadeScaler                 = false;
+    bool EnableTracing                 = false;
+    mfxU32 TraceBufferSize             = 0;
+    SMTTracer::LatencyType LatencyType = SMTTracer::LatencyType::DEFAULT;
+    bool ParallelEncoding              = false;
 
     // session parameters
     bool bIsJoin;
@@ -175,6 +203,11 @@ struct __sInputParams {
     bool isDualMode;
     mfxHyperMode hyperMode;
 #endif
+#if (defined(_WIN32) || defined(_WIN64))
+    //Adapter type
+    bool bPreferiGfx;
+    bool bPreferdGfx;
+#endif
 
     mfxU16 nIdrInterval;
 
@@ -183,7 +216,7 @@ struct __sInputParams {
     mfxI32 dGfxIdx     = -1;
     mfxI32 adapterNum  = -1;
 
-    bool dispFullSearch = false;
+    bool dispFullSearch = DEF_DISP_FULLSEARCH;
 
     mfxU16 nThreadsNum; // number of internal session threads number
     bool bRobustFlag; // Robust transcoding mode. Allows auto-recovery after hardware errors
@@ -197,6 +230,8 @@ struct __sInputParams {
     msdk_char strDumpVppCompFile[MSDK_MAX_FILENAME_LEN]; // VPP composition output dump file
     msdk_char strMfxParamsDumpFile[MSDK_MAX_FILENAME_LEN];
 
+    msdk_char strTCBRCFilePath[MSDK_MAX_FILENAME_LEN];
+
     // specific encode parameters
     mfxU16 nTargetUsage;
     mfxF64 dDecoderFrameRateOverride;
@@ -212,12 +247,17 @@ struct __sInputParams {
     mfxU16 nEncTileRows; // number of rows for encoding tiling
     mfxU16 nEncTileCols; // number of columns for encoding tiling
 
-    bool bEmbeddedDenoiser; // enable or disable embeded HVS Denoiser
-    mfxU16 DenoiseMode;
+    bool
+        bEmbeddedDenoiser; // enable or disable embeded HVS Denoiser (HVS denoiser is integrated into encoder)
+    mfxU16
+        EmbeddedDenoiseMode; // embeded HVS Denoiser Mode (HVS denoiser is integrated into encoder)
+    int EmbeddedDenoiseLevel; // embeded HVS Denoiser Level (HVS denoiser is integrated into encoder)
 
     bool bEnableDeinterlacing;
     mfxU16 DeinterlacingMode;
-    int DenoiseLevel;
+    bool bVppDenoiser;
+    int VppDenoiseLevel;
+    mfxU16 VppDenoiseMode;
     int DetailLevel;
     mfxU16 FRCAlgorithm;
     EFieldCopyMode fieldProcessingMode;
@@ -297,11 +337,16 @@ struct __sInputParams {
     mfxU16 nAvcTemp;
     mfxU16 nBaseLayerPID;
     mfxU16 nAvcTemporalLayers[8];
+#if defined(_WIN32) || defined(_WIN64)
+    mfxU16 bTemporalLayers;
+    mfxTemporalLayer temporalLayers[8];
+#endif
     mfxU16 nSPSId;
     mfxU16 nPPSId;
     mfxU16 nPicTimingSEI;
     mfxU16 nNalHrdConformance;
     mfxU16 nVuiNalHrdParameters;
+    mfxU16 nTransferCharacteristics;
 
     bool bOpenCL;
     mfxU16 reserved[4];
@@ -348,6 +393,14 @@ struct __sInputParams {
     mfxU16 numMFEFrames;
     mfxU16 MFMode;
     mfxU32 mfeTimeout;
+    sSEIMetaMasteringDisplay SEIMetaMDCV;
+    sSEIMetaContentLightLevel SEIMetaCLLI;
+
+    bool bEnableMDCV;
+    bool bEnableCLLI;
+
+    sVideoSignalInfo SignalInfoIn;
+    sVideoSignalInfo SignalInfoOut;
 
     mfxU16 TargetBitDepthLuma;
     mfxU16 TargetBitDepthChroma;
@@ -369,6 +422,10 @@ struct __sInputParams {
 
     bool rawInput;
 
+    // 3DLut Binary File
+    msdk_char str3DLutFile[MSDK_MAX_FILENAME_LEN] = {};
+    bool bEnable3DLut;
+
     mfxU16 nMemoryModel;
 
     mfxPoolAllocationPolicy AllocPolicy = MFX_ALLOCATION_UNLIMITED;
@@ -379,12 +436,16 @@ struct __sInputParams {
 
     mfxU16 nIVFHeader;
 
-    bool IsSourceMSB = false;
+    bool IsSourceMSB      = false;
+    mfxU32 nSyncOpTimeout = MSDK_WAIT_INTERVAL; // SyncOperation timeout in msec
+
+    bool TCBRCFileMode;
 };
 
 struct sInputParams : public __sInputParams {
     sInputParams();
     msdk_string DumpLogFileName;
+
     std::vector<mfxExtEncoderROI> m_ROIData;
 
     bool bDecoderPostProcessing;
@@ -392,6 +453,7 @@ struct sInputParams : public __sInputParams {
 #ifdef ENABLE_MCTF
     sMCTFParam mctfParam;
 #endif
+    eAPIVersion verSessionInit;
 };
 
 static const mfxU32 DecoderTargetID = 100;
@@ -434,16 +496,21 @@ public:
     TargetDescriptor GetDesc(mfxU32 id);
     void PropagateCascadeParameters();
     void CreatePoolList();
+    bool SkipFrame(mfxU32 targetID, mfxU32 frameNum);
 
-    bool ParFileImported       = false;
-    bool CascadeScalerRequired = false;
+    bool ParFileImported          = false;
+    bool CascadeScalerRequired    = false;
+    bool ParallelEncodingRequired = false;
+    mfxU32 GopSize                = 0;
 
     std::vector<TargetDescriptor> Targets;
     std::map<mfxU32, PoolDescritpor> Pools; //key is pool ID
     std::map<mfxU32, sInputParams>
         InParams; //key is target ID, copy of par file for cascade VPP initialization
 
-    SMTTracer* Tracer = nullptr;
+    SMTTracer* Tracer            = nullptr;
+    SMTTracer::PipelineType type = SMTTracer::PipelineType::unknown;
+
     CascadeScalerConfig()
             : ParFileImported(false),
               CascadeScalerRequired(false),
@@ -655,10 +722,13 @@ public:
     virtual ~FileBitstreamProcessor();
     virtual mfxStatus SetReader(std::unique_ptr<CSmplBitstreamReader>& reader);
     virtual mfxStatus SetReader(std::unique_ptr<CSmplYUVReader>& reader);
-    virtual mfxStatus SetWriter(std::unique_ptr<CSmplBitstreamWriter>& writer);
+    virtual mfxStatus SetWriter(std::shared_ptr<CSmplBitstreamWriter>& writer);
     virtual mfxStatus GetInputBitstream(mfxBitstreamWrapper** pBitstream);
     virtual mfxStatus GetInputFrame(mfxFrameSurface1* pSurface);
     virtual mfxStatus ProcessOutputBitstream(mfxBitstreamWrapper* pBitstream);
+    virtual mfxStatus ProcessOutputBitstream(mfxBitstreamWrapper* pBitstream,
+                                             mfxU32 targetID,
+                                             mfxU32 frameNum);
     virtual mfxStatus ResetInput();
     virtual mfxStatus ResetOutput();
     virtual bool IsNulOutput();
@@ -667,7 +737,7 @@ protected:
     std::unique_ptr<CSmplBitstreamReader> m_pFileReader;
     std::unique_ptr<CSmplYUVReader> m_pYUVFileReader;
     // for performance options can be zero
-    std::unique_ptr<CSmplBitstreamWriter> m_pFileWriter;
+    std::shared_ptr<CSmplBitstreamWriter> m_pFileWriter;
     mfxBitstreamWrapper m_Bitstream;
 
 private:
@@ -696,6 +766,7 @@ public:
     // frames allocation is suspended for heterogeneous pipeline
     virtual mfxStatus CompleteInit();
     virtual void Close();
+    virtual mfxStatus Reset(); // for 1.X init
     virtual mfxStatus Reset(VPLImplementationLoader* mfxLoader);
     virtual mfxStatus Join(MFXVideoSession* pChildSession);
     virtual mfxStatus Run();
@@ -728,6 +799,7 @@ public:
     mfxU16 GetNumFrameForAlloc() const;
     bool IsOverlayUsed();
     size_t GetRobustFlag();
+    eAPIVersion GetVersionOfSessionInitAPI();
 
     msdk_string GetSessionText() {
         msdk_stringstream ss;
@@ -740,12 +812,15 @@ public:
     void SetAdapterType(mfxU16 adapterType) {
         m_adapterType = adapterType;
     };
-    void SetPrefferdGfx(mfxI32 dGfxIdx = 0) {
+    void SetPreferdGfx(mfxI32 dGfxIdx = 0) {
         m_dGfxIdx = dGfxIdx;
     };
     void SetAdapterNum(mfxU32 adapterNum = 0) {
         m_adapterNum = adapterNum;
     };
+    void SetSyncOpTimeout(mfxU32 syncOpTimeout = MSDK_WAIT_INTERVAL) {
+        m_nSyncOpTimeout = syncOpTimeout;
+    };
 
     mfxU16 GetAdapterType() const {
         return m_adapterType;
@@ -756,7 +831,25 @@ public:
     mfxI32 GetAdapterNum() const {
         return m_adapterNum;
     };
+    mfxU32 GetSyncOpTimeout() const {
+        return m_nSyncOpTimeout;
+    };
 
+#if (defined(_WIN32) || defined(_WIN64))
+    //Adapter type
+    void SetPreferiGfx(bool preferiGfx) {
+        bPreferiGfx = preferiGfx;
+    };
+    void SetPreferdGfx(bool preferdGfx) {
+        bPreferdGfx = preferdGfx;
+    };
+    bool IsPreferiGfx() {
+        return bPreferiGfx;
+    };
+    bool IsPreferdGfx() {
+        return bPreferdGfx;
+    };
+#endif
 protected:
     virtual mfxStatus CheckRequiredAPIVersion(mfxVersion& version, sInputParams* pParams);
 
@@ -932,6 +1025,7 @@ protected:
     mfxU32 m_nID;
     mfxU16 m_AsyncDepth;
     mfxU32 m_nProcessedFramesNum;
+    mfxU32 m_nTotalFramesNum;
 
     bool m_bIsJoinSession;
 
@@ -940,7 +1034,7 @@ protected:
 
     bool m_bDecodeEnable;
     bool m_bEncodeEnable;
-    mfxU32 m_nVPPCompEnable;
+    mfxU32 m_nVPPCompMode;
     mfxI32 m_libvaBackend;
 
     mfxU16 m_MemoryModel;
@@ -964,7 +1058,6 @@ protected:
     bool m_bInsertIDR;
 
     bool m_rawInput;
-    bool m_shouldUseShifted10BitVPP;
     bool m_shouldUseShifted10BitEnc;
 
     std::unique_ptr<ExtendedBSStore> m_pBSStore;
@@ -1011,6 +1104,10 @@ protected:
 
     void FillMBQPBuffer(mfxExtMBQP& qpMap, mfxU16 pictStruct);
 
+    TCBRCTestFile::Reader m_TCBRCFileReader;
+    bool m_bTCBRCFileMode;
+    mfxStatus ConfigTCBRCTest(mfxFrameSurface1* pSurf);
+
 #ifdef ENABLE_MCTF
     sMctfRunTimeParams m_MctfRTParams;
 #endif
@@ -1020,8 +1117,25 @@ protected:
     mfxI32 m_dGfxIdx     = -1;
     mfxI32 m_adapterNum  = -1;
     mfxU32 TargetID      = 0;
+
     CascadeScalerConfig m_ScalerConfig;
 
+    mfxU32 m_nSyncOpTimeout = MSDK_WAIT_INTERVAL; // SyncOperation timeout in msec
+
+#if (defined(_WIN32) || defined(_WIN64))
+        //Adapter type
+    bool bPreferiGfx;
+    bool bPreferdGfx;
+#endif
+
+    eAPIVersion m_verSessionInit;
+    bool m_b3DLutEnable;
+    // 3DLUT video memory default is 65*65*128*4*2 bytes
+    mfxU32 m_n3DLutVMemId;
+    mfxU32 m_n3DLutVWidth;
+    mfxU32 m_n3DLutVHeight;
+    msdk_char* m_p3DLutFile;
+
 private:
     DISALLOW_COPY_AND_ASSIGN(CTranscodingPipeline);
 };
