@@ -8,8 +8,8 @@
 
 namespace TranscodingSample {
 
-SMTTracer::SMTTracer() : Log(), AddonLog(), E2ELatency(), EncLatency(), TracerFileMutex() {
-    TimeBase = std::chrono::steady_clock::now();
+SMTTracer::SMTTracer() : Log(), AddonLog(), E2ELatency(), EncLatency(), TracerMutex() {
+    TimeBase = GetCurrentTS();
 }
 
 SMTTracer::~SMTTracer() {
@@ -17,22 +17,35 @@ SMTTracer::~SMTTracer() {
         return;
 
     //these functions are intentionally called from destructor to try to save traces in case of a crash
+    AdjustOverlappingEvents();
     AddFlowEvents();
-    SaveTrace(0xffffff & GetCurrentTS());
+    mfxU32 FileID = 0xfffffff & GetCurrentTS();
+    SaveTrace(FileID);
 
     ComputeE2ELatency();
-    PrintE2ELatency();
+    SaveLatency(LatencyType::E2E, FileID, E2ELatency);
 
     ComputeEncLatency();
-    PrintEncLatency();
+    SaveLatency(LatencyType::ENC, FileID, EncLatency);
 }
 
-void SMTTracer::Init() {
+void SMTTracer::Init(const PipelineType type,
+                     const mfxU32 numOfChannels,
+                     const LatencyType latency,
+                     const mfxU32 TraceBufferSize) {
     if (Enabled) {
         return;
     }
     Enabled = true;
+
+    if (TraceBufferSize > TraceBufferSizeInMBytes && TraceBufferSize < MaxTraceBufferSizeInMBytes) {
+        TraceBufferSizeInMBytes = TraceBufferSize;
+    }
     Log.reserve(TraceBufferSizeInMBytes * 1024 * 1024 / sizeof(Event));
+
+    TypeOfPipeline = type;
+    NumOfChannels  = numOfChannels;
+    TypeOfLatency  = latency;
 }
 
 void SMTTracer::BeginEvent(const ThreadType thType,
@@ -64,6 +77,94 @@ void SMTTracer::AddCounterEvent(const ThreadType thType,
     AddEvent(EventType::Counter, thType, thID, name, reinterpret_cast<void*>(counter), nullptr);
 }
 
+void SMTTracer::BeforeDecodeStart() {
+    if (TypeOfLatency == LatencyType::E2E || TypeOfLatency == LatencyType::ENC) {
+        std::unique_lock<std::mutex> guard(TracerMutex);
+        if (NumOfActiveEncoders <= 0) {
+            return;
+        }
+
+        DecSync.wait_for(guard, std::chrono::milliseconds(MaxFrameLatencyInMilliseconds), [&] {
+            return NumOfActiveEncoders <= 0;
+        });
+    }
+}
+
+void SMTTracer::AfterDecodeStart() {
+    if (TypeOfLatency == LatencyType::E2E || TypeOfLatency == LatencyType::ENC) {
+        std::lock_guard<std::mutex> guard(TracerMutex);
+        NumOfActiveEncoders = NumOfChannels;
+        if (TypeOfLatency == LatencyType::ENC) {
+            NumOfActiveDecoders = 1;
+        }
+    }
+}
+
+void SMTTracer::BeforeEncodeStart() {
+    if (TypeOfLatency == LatencyType::ENC) {
+        std::unique_lock<std::mutex> guard(TracerMutex);
+        EncSync.wait_for(guard, std::chrono::milliseconds(MaxFrameLatencyInMilliseconds), [&] {
+            return NumOfActiveDecoders <= 0;
+        });
+    }
+}
+
+void SMTTracer::AfterDecodeSync() {
+    if (TypeOfLatency == LatencyType::ENC) {
+        std::unique_lock<std::mutex> guard(TracerMutex);
+        NumOfActiveDecoders = 0;
+        guard.unlock();
+        EncSync.notify_all();
+    }
+}
+
+void SMTTracer::AfterEncodeSync() {
+    if (TypeOfLatency == LatencyType::E2E || TypeOfLatency == LatencyType::ENC) {
+        std::unique_lock<std::mutex> guard(TracerMutex);
+        NumOfActiveEncoders--;
+
+        if (NumOfActiveEncoders <= 0) {
+            guard.unlock();
+            DecSync.notify_one();
+        }
+    }
+}
+
+void SMTTracer::AdjustOverlappingEvents() {
+    //If two duration events in the same thread overlap, then they or related flow event
+    //may be displayed incorrectly. We move event a bit to avoid overlapping.
+    for (EventIt evABegin = Log.begin(); evABegin != Log.end(); evABegin++) {
+        //find beginning of event A
+        if (evABegin->EvType != EventType::DurationStart) {
+            continue;
+        }
+
+        auto evAEnd   = FindEventInThread(evABegin, EventType::DurationEnd);
+        auto evBBegin = FindEventInThread(evAEnd, EventType::DurationStart);
+        auto evBEnd   = FindEventInThread(evBBegin, EventType::DurationEnd);
+        if (evAEnd == Log.end() || evBBegin == Log.end() || evBEnd == Log.end()) {
+            continue;
+        }
+
+        //now we have two events A and B in the same thread, check if they overlap
+        if (evAEnd->TS == evBBegin->TS) {
+            //events overlap, try to shorten event A
+            if (evABegin->TS != evAEnd->TS) {
+                evAEnd->TS--;
+            }
+            else {
+                //event A is too short already, try to shorten event B
+                if (evBBegin->TS != evBEnd->TS) {
+                    evBBegin->TS++;
+                }
+                else {
+                    //both events are too short already, leave them as is
+                }
+            }
+        }
+    }
+}
+
 void SMTTracer::SaveTrace(mfxU32 FileID) {
     std::string FileName = "smt_trace_" + std::to_string(FileID) + ".json";
     std::ofstream trace_file(FileName, std::ios::out);
@@ -71,10 +172,7 @@ void SMTTracer::SaveTrace(mfxU32 FileID) {
         return;
     }
 
-    printf("\n### trace buffer usage [%d/%d] %.2f%%\n",
-           (int)Log.size(),
-           (int)Log.capacity(),
-           100. * Log.size() / Log.capacity());
+    printf("\n### trace buffer usage %.2f%%\n", 100. * Log.size() / Log.capacity());
     printf("trace file name %s\n", FileName.c_str());
 
     trace_file << "[" << std::endl;
@@ -89,6 +187,11 @@ void SMTTracer::SaveTrace(mfxU32 FileID) {
     trace_file.close();
 }
 
+SMTTracer::TimeInterval::TimeInterval(mfxU64 ts, mfxU64 duration) {
+    TS       = ts;
+    Duration = duration;
+}
+
 void SMTTracer::AddEvent(const EventType evType,
                          const ThreadType thType,
                          const mfxU32 thID,
@@ -108,13 +211,16 @@ void SMTTracer::AddEvent(const EventType evType,
         return;
     }
 
-    std::lock_guard<std::mutex> guard(TracerFileMutex);
+    std::lock_guard<std::mutex> guard(TracerMutex);
     Log.push_back(ev);
 }
 
 mfxU64 SMTTracer::GetCurrentTS() {
-    std::chrono::steady_clock::time_point time = std::chrono::steady_clock::now();
-    return std::chrono::duration_cast<std::chrono::microseconds>(time - TimeBase).count();
+    //Note, that standard specifies that "steady_clock" is monotonic but not system wide,
+    //"system_clock" is system wide but not monotonic. We use "steady_clock", so please
+    //check your implementation before comparing traces from different processes.
+    using namespace std::chrono;
+    return duration_cast<microseconds>(steady_clock::now().time_since_epoch()).count();
 }
 
 void SMTTracer::AddFlowEvents() {
@@ -129,8 +235,9 @@ void SMTTracer::AddFlowEvents() {
 
         auto itc = std::reverse_iterator<decltype(it)>(it);
 
-        auto itp = find_if(itc, Log.rend(), [&it](Event ev) {
-            return ev.EvType == EventType::DurationEnd && it->InID == ev.OutID;
+        auto itp = find_if(itc, Log.rend(), [&it, this](Event ev) {
+            return ev.EvType == EventType::DurationEnd && it->InID == ev.OutID &&
+                   (TypeOfPipeline == PipelineType::_1xN || it->ThID == ev.ThID);
         });
         if (itp == Log.rend()) {
             continue;
@@ -156,21 +263,7 @@ void SMTTracer::ComputeE2ELatency() {
             continue;
         }
 
-        E2ELatency[it->ThID].push_back(it->TS - bit->TS);
-    }
-}
-
-void SMTTracer::PrintE2ELatency() {
-    printf("\nEnd To End Latency (from decode or YUV read start to encode end)\n");
-    printf("    number of frames with unknown latency: %d\n", NumOfErrors);
-
-    for (const auto& v : E2ELatency) {
-        printf("\n    enc%d number of frame %d\n", v.first, int(v.second.size()));
-        printf("        per frame latency ms : ");
-        for (mfxU64 t : v.second) {
-            printf(" %.2f,", t / 1000.);
-        }
-        printf("\n");
+        E2ELatency[it->ThID].push_back(TimeInterval(bit->TS, it->TS - bit->TS));
     }
 }
 
@@ -193,22 +286,70 @@ void SMTTracer::ComputeEncLatency() {
             continue;
         }
 
-        EncLatency[encBegin->ThID].push_back(syncEnd->TS - encBegin->TS);
+        EncLatency[encBegin->ThID].push_back(
+            TimeInterval(encBegin->TS, syncEnd->TS - encBegin->TS));
     }
 }
 
-void SMTTracer::PrintEncLatency() {
-    printf("\nEncode Latency (from encode start to encode end)\n");
-    printf("    number of frames with unknown latency: %d\n", NumOfErrors);
-
-    for (const auto& v : EncLatency) {
-        printf("\n    enc%d number of frame %d\n", v.first, int(v.second.size()));
-        printf("        per frame latency ms : ");
-        for (mfxU64 t : v.second) {
-            printf(" %.2f,", t / 1000.);
-        }
-        printf("\n");
+void SMTTracer::SaveLatency(LatencyType type,
+                            mfxU32 FileID,
+                            std::map<mfxU32, std::vector<TimeInterval>>& latency) {
+    if (latency.empty()) {
+        return;
     }
+
+    std::string FileName = std::string(type == LatencyType::E2E ? "e2e" : "enc") + "_latency_" +
+                           std::to_string(FileID) + ".csv";
+    std::ofstream trace_file(FileName, std::ios::out);
+    if (!trace_file) {
+        return;
+    }
+
+    printf("latency stat file name %s\n", FileName.c_str());
+
+    trace_file << "File format, four columns per channel:" << std::endl;
+    trace_file << "    frame number" << std::endl;
+    trace_file << "    start of frame processing, wall clock, e.g. from system reboot, ms"
+               << std::endl;
+    trace_file << "    start of frame processing, from SMT start, ms" << std::endl;
+    trace_file << "    frame processing duration, ms" << std::endl;
+
+    //header
+    for (const auto& v : latency) {
+        for (int i = 0; i < 4; i++) {
+            trace_file << "enc" << v.first << ",";
+        }
+    }
+    trace_file << std::endl;
+
+    for (size_t i = 0; i < latency.size(); i++) {
+        trace_file << "fr#, ts, ts, latency,";
+    }
+    trace_file << std::endl;
+
+    //data, note, that channels may have different number of frames, for example due to DI
+    size_t NumOfFrames = 0;
+    for (const auto& v : latency) {
+        NumOfFrames = std::max(NumOfFrames, v.second.size());
+    }
+
+    for (size_t i = 0; i < NumOfFrames; i++) {
+        for (const auto& v : latency) {
+            if (v.second.size() > i) {
+                trace_file << i << ",";
+                TimeInterval ti = v.second[i];
+                trace_file << ti.TS / 1000 << ",";
+                trace_file << (ti.TS - TimeBase) / 1000 << ",";
+                trace_file << ti.Duration / 1000. << ",";
+            }
+            else {
+                trace_file << ",,,,";
+            }
+        }
+        trace_file << std::endl;
+    }
+
+    trace_file.close();
 }
 
 SMTTracer::EventIt SMTTracer::FindBeginningOfDependencyChain(EventIt it) {
@@ -257,13 +398,20 @@ SMTTracer::EventIt SMTTracer::FindEndOfPreviosDurationEvent(EventIt it) {
     EventIt itc = it;
     for (; itc != Log.begin();) {
         --itc;
-        if (itc->EvType == EventType::DurationEnd && itc->OutID == it->InID) {
+        if (itc->EvType == EventType::DurationEnd && itc->OutID == it->InID &&
+            (TypeOfPipeline == PipelineType::_1xN || itc->ThID == it->ThID)) {
             return itc;
         }
     }
     return Log.end();
 }
 
+SMTTracer::EventIt SMTTracer::FindEventInThread(EventIt first, EventType type) {
+    return std::find_if(first, Log.end(), [&first, &type](Event ev) {
+        return ev.EvType == type && first->ThType == ev.ThType && first->ThID == ev.ThID;
+    });
+}
+
 void SMTTracer::AddFlowEvent(const Event a, const Event b) {
     if (a.EvType != EventType::DurationEnd || b.EvType != EventType::DurationStart) {
         return;
@@ -367,10 +515,10 @@ void SMTTracer::WriteEventTID(std::ofstream& trace_file, const Event ev) {
     trace_file << "\"tid\":\"";
     switch (ev.ThType) {
         case ThreadType::DEC:
-            trace_file << "dec";
+            trace_file << "dec" << ev.ThID;
             break;
         case ThreadType::VPP:
-            trace_file << "enc" << ev.ThID;
+            trace_file << "enc" << ev.ThID; //it puts VPP events in enc thread
             break;
         case ThreadType::ENC:
             trace_file << "enc" << ev.ThID;
@@ -423,7 +571,7 @@ void SMTTracer::WriteEventName(std::ofstream& trace_file, const Event ev) {
     else if (ev.EvType == EventType::Counter) {
         switch (ev.ThType) {
             case ThreadType::DEC:
-                trace_file << "dec_pool";
+                trace_file << "dec_pool" << ev.ThID;
                 break;
             case ThreadType::VPP:
                 trace_file << "enc_pool" << ev.ThID;
@@ -451,6 +599,9 @@ void SMTTracer::WriteEventName(std::ofstream& trace_file, const Event ev) {
             case EventName::READ_BS:
                 trace_file << "read";
                 break;
+            case EventName::WRITE_BS:
+                trace_file << "write";
+                break;
             case EventName::SURF_WAIT:
                 trace_file << "wait";
                 break;
