@@ -279,10 +279,10 @@ void PrintInfo(sInputParams* pParams, mfxVideoParam* pMfxParams, MFXVideoSession
 
 #ifndef LIBVA_SUPPORT
     if (isHWlib || (pParams->vaType & (ALLOC_IMPL_VIA_D3D9 | ALLOC_IMPL_VIA_D3D11))) {
-        bool isD3D11 = ((ALLOC_IMPL_VIA_D3D11 == pParams->vaType) ||
+        bool isD3D11            = ((ALLOC_IMPL_VIA_D3D11 == pParams->vaType) ||
                         (pParams->ImpLib == (MFX_IMPL_HARDWARE | MFX_IMPL_VIA_D3D11)))
-                           ? true
-                           : false;
+                                      ? true
+                                      : false;
         const msdk_char* sIface = (isD3D11) ? MSDK_STRING("VIA_D3D11") : MSDK_STRING("VIA_D3D9");
         msdk_printf(MSDK_STRING(" | %s"), sIface);
     }
@@ -295,6 +295,14 @@ void PrintInfo(sInputParams* pParams, mfxVideoParam* pMfxParams, MFXVideoSession
     else
         msdk_printf(MSDK_STRING("HW accelaration is disabled\n"));
 
+#if (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+    if (pParams->bPreferdGfx)
+        msdk_printf(MSDK_STRING("dGfx adapter is preferred\n"));
+
+    if (pParams->bPreferiGfx)
+        msdk_printf(MSDK_STRING("iGfx adapter is preferred\n"));
+#endif
+
     mfxVersion ver;
     pMfxSession->QueryVersion(&ver);
     msdk_printf(MSDK_STRING("MediaSDK ver\t%d.%d\n"), ver.Major, ver.Minor);
@@ -417,6 +425,105 @@ mfxStatus InitParamsVPP(MfxVideoParamsWrapper* pParams, sInputParams* pInParams,
     return MFX_ERR_NONE;
 }
 
+#if (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+mfxU32 GetPreferredAdapterNum(const mfxAdaptersInfo& adapters, const sInputParams& params) {
+    if (adapters.NumActual == 0 || !adapters.Adapters)
+        return 0;
+
+    if (params.bPreferdGfx) {
+        // Find dGfx adapter in list and return it's index
+
+        auto idx = std::find_if(adapters.Adapters,
+                                adapters.Adapters + adapters.NumActual,
+                                [](const mfxAdapterInfo info) {
+                                    return info.Platform.MediaAdapterType ==
+                                           mfxMediaAdapterType::MFX_MEDIA_DISCRETE;
+                                });
+
+        // No dGfx in list
+        if (idx == adapters.Adapters + adapters.NumActual) {
+            msdk_printf(
+                MSDK_STRING("Warning: No dGfx detected on machine. Will pick another adapter\n"));
+            return 0;
+        }
+
+        return static_cast<mfxU32>(std::distance(adapters.Adapters, idx));
+    }
+
+    if (params.bPreferiGfx) {
+        // Find iGfx adapter in list and return it's index
+
+        auto idx = std::find_if(adapters.Adapters,
+                                adapters.Adapters + adapters.NumActual,
+                                [](const mfxAdapterInfo info) {
+                                    return info.Platform.MediaAdapterType ==
+                                           mfxMediaAdapterType::MFX_MEDIA_INTEGRATED;
+                                });
+
+        // No iGfx in list
+        if (idx == adapters.Adapters + adapters.NumActual) {
+            msdk_printf(
+                MSDK_STRING("Warning: No iGfx detected on machine. Will pick another adapter\n"));
+            return 0;
+        }
+
+        return static_cast<mfxU32>(std::distance(adapters.Adapters, idx));
+    }
+
+    // Other ways return 0, i.e. best suitable detected by dispatcher
+    return 0;
+}
+
+mfxStatus GetImpl(const mfxVideoParam& params, mfxIMPL& impl, const sInputParams& cmd_params) {
+    if (!(impl & MFX_IMPL_HARDWARE))
+        return MFX_ERR_NONE;
+
+    mfxU32 num_adapters_available;
+
+    mfxStatus sts = MFXQueryAdaptersNumber(&num_adapters_available);
+    MSDK_CHECK_STATUS(sts, "MFXQueryAdaptersNumber failed");
+
+    mfxComponentInfo interface_request;
+    memset(&interface_request, 0, sizeof(interface_request));
+    interface_request.Type             = mfxComponentType::MFX_COMPONENT_VPP;
+    interface_request.Requirements.vpp = params.vpp;
+
+    std::vector<mfxAdapterInfo> displays_data(num_adapters_available);
+    mfxAdaptersInfo adapters = { displays_data.data(), mfxU32(displays_data.size()), 0u };
+
+    sts = MFXQueryAdapters(&interface_request, &adapters);
+    if (sts == MFX_ERR_NOT_FOUND) {
+        msdk_printf(MSDK_STRING("ERROR: No suitable adapters found for this workload\n"));
+    }
+    MSDK_CHECK_STATUS(sts, "MFXQueryAdapters failed");
+
+    impl &= ~MFX_IMPL_HARDWARE;
+
+    mfxU32 idx = GetPreferredAdapterNum(adapters, cmd_params);
+    switch (adapters.Adapters[idx].Number) {
+        case 0:
+            impl |= MFX_IMPL_HARDWARE;
+            break;
+        case 1:
+            impl |= MFX_IMPL_HARDWARE2;
+            break;
+        case 2:
+            impl |= MFX_IMPL_HARDWARE3;
+            break;
+        case 3:
+            impl |= MFX_IMPL_HARDWARE4;
+            break;
+
+        default:
+            // Try searching on all display adapters
+            impl |= MFX_IMPL_HARDWARE_ANY;
+            break;
+    }
+
+    return MFX_ERR_NONE;
+}
+#endif // (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+
 /* ******************************************************************* */
 
 mfxStatus CreateFrameProcessor(sFrameProcessor* pProcessor,
@@ -431,40 +538,59 @@ mfxStatus CreateFrameProcessor(sFrameProcessor* pProcessor,
     WipeFrameProcessor(pProcessor);
 
     //MFX session
-    pProcessor->pLoader.reset(new VPLImplementationLoader);
+    if (pInParams->verSessionInit == API_1X) {
+#if (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+        sts = GetImpl(*pParams, impl, *pInParams);
+        MSDK_CHECK_STATUS(sts, "GetImpl failed");
+#endif
+        mfxVersion version = { { 10, 1 } };
+        mfxInitParamlWrap initParams;
+        initParams.ExternalThreads = 0;
+        initParams.GPUCopy         = pInParams->GPUCopyValue;
+        initParams.Implementation  = impl;
+        initParams.Version         = version;
+        initParams.NumExtParam     = 0;
+        sts                        = pProcessor->mfxSession.InitEx(initParams);
 
-    if (pInParams->dGfxIdx >= 0)
-        pProcessor->pLoader->SetDiscreteAdapterIndex(pInParams->dGfxIdx);
-    else
-        pProcessor->pLoader->SetAdapterType(pInParams->adapterType);
+        MSDK_CHECK_STATUS_SAFE(sts, "pProcessor->mfxSession.Init failed", {
+            WipeFrameProcessor(pProcessor);
+        });
+    }
+    else {
+        pProcessor->pLoader.reset(new VPLImplementationLoader);
 
-    if (pInParams->adapterNum >= 0)
-        pProcessor->pLoader->SetAdapterNum(pInParams->adapterNum);
+        if (pInParams->dGfxIdx >= 0)
+            pProcessor->pLoader->SetDiscreteAdapterIndex(pInParams->dGfxIdx);
+        else
+            pProcessor->pLoader->SetAdapterType(pInParams->adapterType);
+
+        if (pInParams->adapterNum >= 0)
+            pProcessor->pLoader->SetAdapterNum(pInParams->adapterNum);
 
 #ifdef ONEVPL_EXPERIMENTAL
-    if (pInParams->PCIDeviceSetup)
-        pProcessor->pLoader->SetPCIDevice(pInParams->PCIDomain,
-                                          pInParams->PCIBus,
-                                          pInParams->PCIDevice,
-                                          pInParams->PCIFunction);
+        if (pInParams->PCIDeviceSetup)
+            pProcessor->pLoader->SetPCIDevice(pInParams->PCIDomain,
+                                              pInParams->PCIBus,
+                                              pInParams->PCIDevice,
+                                              pInParams->PCIFunction);
 
     #if (defined(_WIN64) || defined(_WIN32))
-    if (pInParams->luid.HighPart > 0 || pInParams->luid.LowPart > 0)
-        pProcessor->pLoader->SetupLUID(pInParams->luid);
+        if (pInParams->luid.HighPart > 0 || pInParams->luid.LowPart > 0)
+            pProcessor->pLoader->SetupLUID(pInParams->luid);
     #else
-    pProcessor->pLoader->SetupDRMRenderNodeNum(pInParams->DRMRenderNodeNum);
+        pProcessor->pLoader->SetupDRMRenderNodeNum(pInParams->DRMRenderNodeNum);
     #endif
 #endif
 
-    bool bLowLatencyMode = !pInParams->dispFullSearch;
-
-    sts = pProcessor->pLoader->ConfigureAndEnumImplementations(impl,
-                                                               pInParams->accelerationMode,
-                                                               bLowLatencyMode);
-    MSDK_CHECK_STATUS(sts, "mfxSession.EnumImplementations failed");
-    sts = pProcessor->mfxSession.CreateSession(pProcessor->pLoader.get());
-    MSDK_CHECK_STATUS(sts, "m_mfxSession.CreateSession failed");
+        bool bLowLatencyMode = !pInParams->dispFullSearch;
 
+        sts = pProcessor->pLoader->ConfigureAndEnumImplementations(impl,
+                                                                   pInParams->accelerationMode,
+                                                                   bLowLatencyMode);
+        MSDK_CHECK_STATUS(sts, "mfxSession.EnumImplementations failed");
+        sts = pProcessor->mfxSession.CreateSession(pProcessor->pLoader.get());
+        MSDK_CHECK_STATUS(sts, "m_mfxSession.CreateSession failed");
+    }
     // VPP
     pProcessor->pmfxVPP = new MFXVideoVPP(pProcessor->mfxSession);
 
@@ -546,8 +672,12 @@ mfxStatus InitMemoryAllocator(sFrameProcessor* pProcessor,
 #ifdef D3D_SURFACES_SUPPORT
             // prepare device manager
             pAllocator->pDevice = new CD3D9Device();
-            sts =
-                pAllocator->pDevice->Init(0, 1, MSDKAdapter::GetNumber(pProcessor->pLoader.get()));
+
+            mfxU32 adapterNum = (pInParams->verSessionInit == API_1X)
+                                    ? MSDKAdapter::GetNumber(pProcessor->mfxSession)
+                                    : MSDKAdapter::GetNumber(pProcessor->pLoader.get());
+
+            sts = pAllocator->pDevice->Init(0, 1, adapterNum);
             MSDK_CHECK_STATUS_SAFE(sts,
                                    "pAllocator->pDevice->Init failed",
                                    WipeMemoryAllocator(pAllocator));
@@ -573,8 +703,11 @@ mfxStatus InitMemoryAllocator(sFrameProcessor* pProcessor,
 #if MFX_D3D11_SUPPORT
             pAllocator->pDevice = new CD3D11Device();
 
-            sts =
-                pAllocator->pDevice->Init(0, 1, MSDKAdapter::GetNumber(pProcessor->pLoader.get()));
+            mfxU32 adapterNum = (pInParams->verSessionInit == API_1X)
+                                    ? MSDKAdapter::GetNumber(pProcessor->mfxSession)
+                                    : MSDKAdapter::GetNumber(pProcessor->pLoader.get());
+
+            sts = pAllocator->pDevice->Init(0, 1, adapterNum);
             MSDK_CHECK_STATUS_SAFE(sts,
                                    "pAllocator->pDevice->Init failed",
                                    WipeMemoryAllocator(pAllocator));
@@ -601,8 +734,11 @@ mfxStatus InitMemoryAllocator(sFrameProcessor* pProcessor,
             pAllocator->pDevice = CreateVAAPIDevice(pInParams->strDevicePath);
             MSDK_CHECK_POINTER(pAllocator->pDevice, MFX_ERR_NULL_PTR);
 
-            sts =
-                pAllocator->pDevice->Init(0, 1, MSDKAdapter::GetNumber(pProcessor->pLoader.get()));
+            mfxU32 adapterNum = (pInParams->verSessionInit == API_1X)
+                                    ? MSDKAdapter::GetNumber(pProcessor->mfxSession)
+                                    : MSDKAdapter::GetNumber(pProcessor->pLoader.get());
+
+            sts = pAllocator->pDevice->Init(0, 1, adapterNum);
             MSDK_CHECK_STATUS_SAFE(sts,
                                    "pAllocator->pDevice->Init failed",
                                    WipeMemoryAllocator(pAllocator));
@@ -1292,6 +1428,58 @@ mfxStatus CRawVideoReader::LoadNextFrame(mfxFrameData* pData, mfxFrameInfo* pInf
     return MFX_ERR_NONE;
 }
 
+mfxStatus CRawVideoReader::LoadNextFrame(mfxFrameSurface1* pSurface,
+                                         int bytes_to_read,
+                                         mfxU8* buf_read) {
+    // check if reader is initialized
+    MSDK_CHECK_POINTER(pSurface, MFX_ERR_NULL_PTR);
+
+    int nBytesRead = static_cast<int>(fread(buf_read, 1, bytes_to_read, m_fSrc));
+
+    if (bytes_to_read != nBytesRead) {
+        return MFX_ERR_MORE_DATA;
+    }
+
+    mfxU16 w, h;
+    mfxFrameInfo* pInfo = &pSurface->Info;
+    mfxFrameData* pData = &pSurface->Data;
+
+    w = pInfo->Width;
+    h = pInfo->Height;
+
+    switch (pInfo->FourCC) {
+        case MFX_FOURCC_NV12:
+            pData->Y  = buf_read;
+            pData->UV = pData->Y + w * h;
+            break;
+        case MFX_FOURCC_I420:
+            pData->Y = buf_read;
+            pData->U = pData->Y + w * h;
+            pData->V = pData->U + ((w / 2) * (h / 2));
+            break;
+
+        case MFX_FOURCC_P010:
+            pData->Y  = buf_read;
+            pData->UV = pData->Y + w * 2 * h;
+            break;
+        case MFX_FOURCC_I010:
+            pData->Y = buf_read;
+            pData->U = pData->Y + w * 2 * h;
+            pData->V = pData->U + (w * (h / 2));
+            break;
+
+        case MFX_FOURCC_RGB4:
+            // read luminance plane (Y)
+            //pitch    = pData->Pitch;
+            pData->B = buf_read;
+            break;
+        default:
+            break;
+    }
+
+    return MFX_ERR_NONE;
+}
+
 mfxStatus CRawVideoReader::GetNextInputFrame(sMemoryAllocator* pAllocator,
                                              mfxFrameInfo* pInfo,
                                              mfxFrameSurfaceWrap** pSurface,
@@ -1335,6 +1523,31 @@ mfxStatus CRawVideoReader::GetNextInputFrame(sMemoryAllocator* pAllocator,
     return MFX_ERR_NONE;
 }
 
+mfxStatus CRawVideoReader::GetNextInputFrame(sFrameProcessor* pProcessor,
+                                             mfxFrameInfo* pInfo,
+                                             mfxFrameSurfaceWrap** pSurface,
+                                             int bytes_to_read,
+                                             mfxU8* buf_read) {
+    mfxStatus sts;
+    sts = pProcessor->mfxSession.GetSurfaceForVPP((mfxFrameSurface1**)pSurface);
+    MSDK_CHECK_STATUS(sts, "GetSurfaceForVPPIn failed");
+
+    // Map makes surface writable by CPU for all implementations
+    sts = (*pSurface)->FrameInterface->Map(*pSurface, MFX_MAP_WRITE);
+    MSDK_CHECK_STATUS(sts, "mfxFrameSurfaceInterface->Map failed");
+
+    sts = LoadNextFrame(*pSurface, bytes_to_read, buf_read);
+
+    // Unmap/release returns local device access for all implementations
+    mfxStatus lsts = (*pSurface)->FrameInterface->Unmap(*pSurface);
+    MSDK_CHECK_STATUS(lsts, "mfxFrameSurfaceInterface->Unmap failed");
+
+    lsts = (*pSurface)->FrameInterface->Release(*pSurface);
+    MSDK_CHECK_STATUS(lsts, "mfxFrameSurfaceInterface->Release failed");
+
+    return sts;
+}
+
 mfxStatus CRawVideoReader::GetPreAllocFrame(mfxFrameSurfaceWrap** pSurface) {
     if (m_it == m_SurfacesList.end()) {
         m_Repeat--;
@@ -1472,6 +1685,26 @@ mfxStatus CRawVideoWriter::PutNextFrame(sMemoryAllocator* pAllocator,
 
     return MFX_ERR_NONE;
 }
+
+mfxStatus CRawVideoWriter::PutNextFrame(mfxFrameInfo* pInfo, mfxFrameSurfaceWrap* pSurface) {
+    mfxStatus sts;
+    if (m_fDst) {
+        sts = pSurface->FrameInterface->Map(pSurface, MFX_MAP_READ);
+        MSDK_CHECK_NOT_EQUAL(sts, MFX_ERR_NONE, MFX_ERR_ABORTED);
+
+        sts = WriteFrame(&(pSurface->Data), pInfo);
+        MSDK_CHECK_NOT_EQUAL(sts, MFX_ERR_NONE, MFX_ERR_ABORTED);
+
+        sts = pSurface->FrameInterface->Unmap(pSurface);
+        MSDK_CHECK_NOT_EQUAL(sts, MFX_ERR_NONE, MFX_ERR_ABORTED);
+    }
+
+    sts = pSurface->FrameInterface->Release(pSurface);
+    MSDK_CHECK_NOT_EQUAL(sts, MFX_ERR_NONE, MFX_ERR_ABORTED);
+
+    return sts;
+}
+
 mfxStatus CRawVideoWriter::WriteFrame(mfxFrameData* pData, mfxFrameInfo* pInfo) {
     mfxI32 nBytesRead = 0;
 
@@ -2003,6 +2236,15 @@ mfxStatus GeneralWriter::PutNextFrame(sMemoryAllocator* pAllocator,
     return sts;
 };
 
+mfxStatus GeneralWriter::PutNextFrame(mfxFrameInfo* pInfo, mfxFrameSurfaceWrap* pSurface) {
+    mfxU32 did = (m_svcMode) ? pSurface->Info.FrameId.DependencyId
+                             : 0; //aya: for MVC we have 1 out file only
+
+    mfxStatus sts = m_ofile[did]->PutNextFrame(pInfo, pSurface);
+
+    return sts;
+};
+
 /* ******************************************************************* */
 
 mfxStatus UpdateSurfacePool(mfxFrameInfo SurfacesInfo,
